@@ -1,4 +1,4 @@
-/* $Id: ml_gl.c,v 1.41 2003/10/03 22:52:19 garrigue Exp $ */
+/* $Id: ml_gl.c,v 1.43 2004/04/16 11:07:22 raffalli Exp $ */
 
 #ifdef _WIN32
 #include <wtypes.h>
@@ -154,7 +154,19 @@ ML_1 (glDisable, GLenum_val)
 ML_1 (glDepthFunc, GLenum_val)
 ML_1 (glDepthMask, Int_val)
 ML_2 (glDepthRange, Double_val, Double_val)
-ML_1 (glDrawBuffer, GLenum_val)
+
+CAMLprim value ml_glDrawBuffer (value buffer)
+{
+    if (Is_block(buffer)) {
+	int n = Int_val (Field(buffer,1));
+	if (n >= GL_AUX_BUFFERS)
+	    ml_raise_gl ("GlFunc.draw_buffer : no such auxiliary buffer");
+	glDrawBuffer (GL_AUX0 + n);
+    }
+    else glDrawBuffer (GLenum_val(buffer));
+    return Val_unit;
+}
+
 ML_4 (glDrawPixels, Int_val, Int_val, GLenum_val, Type_void_raw)
 
 ML_1 (glEdgeFlag, Int_val)
@@ -506,15 +518,21 @@ CAMLprim value ml_glReadBuffer (value buffer)
     if (Is_block(buffer)) {
 	int n = Int_val (Field(buffer,1));
 	if (n >= GL_AUX_BUFFERS)
-	    ml_raise_gl ("Gl.read_buffer : no such auxiliary buffer");
+	    ml_raise_gl ("GlFunc.read_buffer : no such auxiliary buffer");
 	glReadBuffer (GL_AUX0 + n);
     }
     else glReadBuffer (GLenum_val(buffer));
     return Val_unit;
 }
 
-ML_6 (glReadPixels, Int_val, Int_val, Int_val, Int_val, GLenum_val,
-      Type_void_raw)
+CAMLprim value ml_glReadPixels(value x, value y, value w, value h, value format , value raw)  /* ML */
+{
+  glPixelStorei(GL_PACK_SWAP_BYTES, 0);
+  glPixelStorei(GL_PACK_ALIGNMENT, 1);
+  glReadPixels(Int_val(x),Int_val(y),Int_val(w),Int_val(h),GLenum_val(format),
+	       Type_void_raw(raw));
+}
+
 ML_bc6 (ml_glReadPixels)
 ML_2 (glRectd, Pair(arg1,Double_val,Double_val),
       Pair(arg2,Double_val,Double_val))
